import { Subject } from 'rxjs';
import { HttpClient } from '@angular/common/http';
import { map } from 'rxjs/operators';
import { DomSanitizer } from '@angular/platform-browser';
import { Injectable, Optional, NgModule, Component, Input, defineInjectable, inject } from '@angular/core';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgScormPlayerConfig {
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ScormResult {
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgScormPlayerService {
    /**
     * @param {?} config
     * @param {?} http
     */
    constructor(config, http) {
        this.http = http;
        this.closeWindowOnFinish = false;
        this.errorLogs = [];
        this.openInNewWindow = false;
        // Events
        this.initializeSource = new Subject();
        this.initializeEvent = this.initializeSource.asObservable();
        this.setValueSource = new Subject();
        this.setValueEvent = this.setValueSource.asObservable();
        this.getValueSource = new Subject();
        this.getValueEvent = this.getValueSource.asObservable();
        this.finishSource = new Subject();
        this.finishEvent = this.finishSource.asObservable();
        this.commitSource = new Subject();
        this.commitEvent = this.commitSource.asObservable();
        if (config) {
            this.debug = config.debug;
            this.openInNewWindow = config.openInNewWindow;
        }
    }
    // HELPERS
    /**
     * read the manifest and get the url launch page
     * @param {?} urlDirSco url of the directory of the scorm
     * @return {?}
     */
    getUrlLaunchPage(urlDirSco) {
        /** @type {?} */
        const url = `${urlDirSco}/imsmanifest.xml`;
        return this.http.get(url, { responseType: 'text' }).pipe(map((/**
         * @param {?} res
         * @return {?}
         */
        res => {
            /** @type {?} */
            let launchUrl = '';
            /** @type {?} */
            const parser = new DOMParser();
            /** @type {?} */
            const doc = parser.parseFromString(res, 'text/xml');
            /** @type {?} */
            const resources = doc.getElementsByTagName('resource');
            /** @type {?} */
            const filename = resources[0].getAttribute('href');
            launchUrl = `${urlDirSco}/${filename}`;
            return launchUrl;
        })));
    }
    // SCORM 2004
    /**
     * @return {?}
     */
    Initialize() {
        return this.MyInitialize();
    }
    /**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    SetValue(key, value) {
        return this.MySetValue(key, value);
    }
    /**
     * @param {?} key
     * @return {?}
     */
    GetValue(key) {
        return this.MyGetValue(key);
    }
    /**
     * @return {?}
     */
    Terminate() {
        return this.MyFinish(); // return false to avoid refreh in some  cases
    }
    /**
     * @return {?}
     */
    Commit() {
        return this.MyCommit();
    }
    /**
     * @return {?}
     */
    GetLastError() {
        return this.MyGetLastError();
    }
    /**
     * @param {?} errorCode
     * @return {?}
     */
    GetErrorString(errorCode) {
        return this.MyGetErrorString(errorCode);
    }
    /**
     * @param {?} errorCode
     * @return {?}
     */
    GetDiagnostic(errorCode) {
        return this.MyGetDiagnostic(errorCode);
    }
    // SCORM 1.2
    /**
     * @return {?}
     */
    LMSInitialize() {
        return this.MyInitialize();
    }
    /**
     * @param {?} data
     * @return {?}
     */
    LMSFinish(data) {
        return this.MyFinish(); // return false to avoid refreh in some  cases
    }
    /**
     * @param {?} key
     * @return {?}
     */
    LMSGetValue(key) {
        return this.MyGetValue(key);
    }
    /**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    LMSSetValue(key, value) {
        return this.MySetValue(key, value);
    }
    /**
     * @return {?}
     */
    LMSCommit() {
        return this.MyCommit();
    }
    /**
     * @return {?}
     */
    LMSGetLastError() {
        return this.MyGetLastError();
    }
    /**
     * @param {?} errorCode
     * @return {?}
     */
    LMSGetErrorString(errorCode) {
        return this.MyGetErrorString(errorCode);
    }
    /**
     * @param {?} errorCode
     * @return {?}
     */
    LMSGetDiagnostic(errorCode) {
        return this.MyGetDiagnostic(errorCode);
    }
    // My Method
    /**
     * @return {?}
     */
    MyInitialize() {
        try {
            if (!this.runtimeData) {
                this.runtimeData = {};
            }
            if (!this.scormResult) {
                this.scormResult = new ScormResult();
            }
            this.initializeSource.next(this.scormResult);
            return true;
        }
        catch (err) {
            this.logErrorCode(3);
            return false;
        }
    }
    /**
     * @return {?}
     */
    MyFinish() {
        try {
            if (this.runtimeData['cmi.completion_status'] === 'completed') {
                this.scormResult.completedOn = new Date();
            }
            this.scormResult.timeSpent = this.parseDurationString(this.runtimeData['cmi.session_time']);
            // progess mesure is value between 0 and 1
            this.scormResult.progression = this.runtimeData['cmi.progress_measure'] ? this.runtimeData['cmi.progress_measure'] * 100 : null;
            this.scormResult.scorePercent = this.runtimeData['cmi.score.scaled'] ? this.runtimeData['cmi.score.scaled'] * 100 : null;
            this.scormResult.runtimeData = this.runtimeData;
            this.finishSource.next(this.scormResult);
            return true;
        }
        catch (err) {
            this.logErrorCode(5);
            return true;
        }
    }
    /**
     * @param {?} key
     * @return {?}
     */
    MyGetValue(key) {
        try {
            this.getValueSource.next(this.scormResult);
            return this.runtimeData[key] ? this.runtimeData[key] : '';
        }
        catch (err) {
            this.logErrorCode(1);
            return '';
        }
    }
    /**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    MySetValue(key, value) {
        try {
            this.runtimeData[key] = value;
            this.scormResult.runtimeData = this.runtimeData;
            this.setValueSource.next(this.scormResult);
            return value;
        }
        catch (err) {
            this.logErrorCode(2);
            return value;
        }
    }
    /**
     * @return {?}
     */
    MyCommit() {
        try {
            this.scormResult.timeSpent = this.parseDurationString(this.runtimeData['cmi.session_time']);
            // progess mesure is value between 0 and 1
            this.scormResult.progression = this.runtimeData['cmi.progress_measure'] ? this.runtimeData['cmi.progress_measure'] * 100 : null;
            this.scormResult.scorePercent = this.runtimeData['cmi.score.scaled'] ? this.runtimeData['cmi.score.scaled'] * 100 : null;
            this.scormResult.runtimeData = this.runtimeData;
            this.commitSource.next(this.scormResult);
            return true;
        }
        catch (err) {
            this.logErrorCode(6);
            return false;
        }
    }
    /**
     * @return {?}
     */
    MyGetLastError() {
        return this.errorLogs.length > 0 ? this.errorLogs[0] : 0;
    }
    /**
     * @param {?} errorCode
     * @return {?}
     */
    MyGetErrorString(errorCode) {
        /** @type {?} */
        let errorString;
        switch (errorCode) {
            case 0:
                errorString = 'None error';
                break;
            case 1:
                errorString = 'Error to get value';
                break;
            case 2:
                errorString = 'Error to set value';
                break;
            case 3:
                errorString = 'Error to Initialize';
                break;
            case 4:
                errorString = 'Error to save data';
                break;
            case 5:
                errorString = 'Error to Finish';
                break;
            case 6:
                errorString = 'Error to Commit';
                break;
        }
        return errorString;
    }
    /**
     * @param {?} errocCode
     * @return {?}
     */
    MyGetDiagnostic(errocCode) {
        return 'NO_DIAGNOSTIC';
    }
    // helpers
    /**
     * @private
     * @param {?} msg
     * @return {?}
     */
    logErrorCode(msg) {
        this.errorLogs.push(msg);
    }
    /**
     * @private
     * @param {?} durationString
     * @return {?}
     */
    parseDurationString(durationString) {
        if (!durationString || durationString === '') {
            return 0;
        }
        /** @type {?} */
        const stringPattern = /^PT(?:(\d+)D)?(?:(\d+)H)?(?:(\d+)M)?(?:(\d+(?:\.\d{1,3})?)S)?$/;
        /** @type {?} */
        const stringParts = stringPattern.exec(durationString);
        return ((((stringParts[1] === undefined ? 0 : Number(stringParts[1])) /* Days */
            * 24 + (stringParts[2] === undefined ? 0 : Number(stringParts[2])) /* Hours */)
            * 60 + (stringParts[3] === undefined ? 0 : Number(stringParts[3])) /* Minutes */)
            * 60 + (stringParts[4] === undefined ? 0 : Number(stringParts[4])) /* Seconds */);
    }
}
NgScormPlayerService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
NgScormPlayerService.ctorParameters = () => [
    { type: NgScormPlayerConfig, decorators: [{ type: Optional }] },
    { type: HttpClient }
];
/** @nocollapse */ NgScormPlayerService.ngInjectableDef = defineInjectable({ factory: function NgScormPlayerService_Factory() { return new NgScormPlayerService(inject(NgScormPlayerConfig, 8), inject(HttpClient)); }, token: NgScormPlayerService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @return {?}
 */
function _window() {
    // return the global native browser window object
    return window;
}
class WindowRef {
    /**
     * @return {?}
     */
    get nativeWindow() {
        return _window();
    }
}
WindowRef.decorators = [
    { type: Injectable }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgScormPlayerComponent {
    /**
     * @param {?} win
     * @param {?} playerSco
     * @param {?} sanitizer
     */
    constructor(win, playerSco, sanitizer) {
        this.win = win;
        this.playerSco = playerSco;
        this.sanitizer = sanitizer;
        this.openInNewWindow = this.openInNewWindow ? this.openInNewWindow : this.playerSco.openInNewWindow;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.init();
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        this.init();
        if (this.playerSco.debug) {
            console.log('prop changes:', changes);
        }
    }
    /**
     * @return {?}
     */
    init() {
        if (this.initializing) {
            return;
        }
        this.initializing = true;
        this.win.nativeWindow.API_1484_11 = this.playerSco;
        this.win.nativeWindow.API = this.playerSco;
        // try to set the api directly in the frame window
        try {
            /** @type {?} */
            var frame = document.getElementById("scoIframe");
            frame['contentWindow'].API_1484_11 = this.playerSco;
            frame['contentWindow'].API = this.playerSco;
        }
        catch (err) {
            if (this.playerSco.debug) {
                console.log('error settings api directlty in frame:', err);
            }
        }
        if (!this.urlLaunchPage && this.urlDirSco) {
            this.playerSco.getUrlLaunchPage(this.urlDirSco).subscribe((/**
             * @param {?} url
             * @return {?}
             */
            url => {
                this.urlLaunchPageSafe = this.sanitizer.bypassSecurityTrustResourceUrl(url);
                this.maybeOpenWindow(url);
                this.initializing = false;
            }), (/**
             * @param {?} err
             * @return {?}
             */
            err => {
                if (this.playerSco.debug) {
                    console.log('error finding urlLaunchPage', err);
                }
                this.initializing = false;
            }));
        }
        else if (this.urlLaunchPage) {
            this.urlLaunchPageSafe = this.sanitizer.bypassSecurityTrustResourceUrl(this.urlLaunchPage);
            this.maybeOpenWindow(this.urlLaunchPage);
            this.initializing = false;
        }
        else {
            if (this.playerSco.debug) {
                console.error('no url provided');
            }
            this.initializing = false;
        }
    }
    /**
     * @param {?} url
     * @return {?}
     */
    maybeOpenWindow(url) {
        if (this.openInNewWindow) {
            this.win.nativeWindow.open(url, '_blank');
        }
    }
}
NgScormPlayerComponent.decorators = [
    { type: Component, args: [{
                selector: 'ng-scorm-player',
                template: "<div class=\"container\" *ngIf=\"urlLaunchPageSafe\">\n  <iframe *ngIf=\"!openInNewWindow\"\n  class=\"no-border\" \n  sandbox=\"allow-scripts allow-forms allow-same-origin allow-modals allow-top-navigation allow-popups\" \n  [src]=\"urlLaunchPageSafe\" \n  name=\"scoIframe\" \n  id=\"scoIframe\" \n  width=\"100%\" \n  height=\"100%\" >    \n  </iframe>\n</div>\n\n\n",
                styles: [".container{height:100%;width:100%}.no-border{border:none!important}"]
            }] }
];
/** @nocollapse */
NgScormPlayerComponent.ctorParameters = () => [
    { type: WindowRef },
    { type: NgScormPlayerService },
    { type: DomSanitizer }
];
NgScormPlayerComponent.propDecorators = {
    urlDirSco: [{ type: Input }],
    urlLaunchPage: [{ type: Input }],
    scormResult: [{ type: Input }],
    openInNewWindow: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgScormPlayerModule {
    /**
     * @param {?} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: NgScormPlayerModule,
            providers: [
                { provide: NgScormPlayerConfig, useValue: config }
            ]
        };
    }
    /**
     * @param {?} config
     * @return {?}
     */
    static forChild(config) {
        return {
            ngModule: NgScormPlayerModule,
            providers: [
                { provide: NgScormPlayerConfig, useValue: config }
            ],
        };
    }
}
NgScormPlayerModule.decorators = [
    { type: NgModule, args: [{
                declarations: [NgScormPlayerComponent],
                imports: [
                    CommonModule
                ],
                exports: [
                    NgScormPlayerComponent
                ],
                providers: [
                    WindowRef
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgScormPlayerConfig, ScormResult, NgScormPlayerService, NgScormPlayerComponent, NgScormPlayerModule, WindowRef as ɵa };

//# sourceMappingURL=ng-scorm-player.js.map